@@ -30,6 +30,7 @@
 #include "libavutil/internal.h"
 #include "avcodec.h"
 #include "error_resilience.h"
+#include "mpegutils.h"
 #include "mpegvideo.h"
 #include "rectangle.h"
 #include "thread.h"
@@ -57,7 +58,7 @@ static void set_mv_strides(ERContext *s, int *mv_step, int *stride)
 static void put_dc(ERContext *s, uint8_t *dest_y, uint8_t *dest_cb,
                    uint8_t *dest_cr, int mb_x, int mb_y)
 {
-    int *linesize = s->cur_pic->f.linesize;
+    int *linesize = s->cur_pic.f->linesize;
     int dc, dcu, dcv, y, i;
     for (i = 0; i < 4; i++) {
         dc = s->dc_val[0][mb_x * 2 + (i &  1) + (mb_y * 2 + (i >> 1)) * s->b8_stride];
@@ -135,8 +136,8 @@ static void guess_dc(ERContext *s, int16_t *dc, int w,
                      int h, int stride, int is_luma)
 {
     int b_x, b_y;
-    int16_t  (*col )[4] = av_malloc(stride*h*sizeof( int16_t)*4);
-    uint32_t (*dist)[4] = av_malloc(stride*h*sizeof(uint32_t)*4);
+    int16_t  (*col )[4] = av_malloc_array(stride, h*sizeof( int16_t)*4);
+    uint32_t (*dist)[4] = av_malloc_array(stride, h*sizeof(uint32_t)*4);
 
     if(!col || !dist) {
         av_log(s->avctx, AV_LOG_ERROR, "guess_dc() is out of memory\n");
@@ -149,7 +150,7 @@ static void guess_dc(ERContext *s, int16_t *dc, int w,
         for(b_x=0; b_x<w; b_x++){
             int mb_index_j= (b_x>>is_luma) + (b_y>>is_luma)*s->mb_stride;
             int error_j= s->error_status_table[mb_index_j];
-            int intra_j = IS_INTRA(s->cur_pic->mb_type[mb_index_j]);
+            int intra_j = IS_INTRA(s->cur_pic.mb_type[mb_index_j]);
             if(intra_j==0 || !(error_j&ER_DC_ERROR)){
                 color= dc[b_x + b_y*stride];
                 distance= b_x;
@@ -162,7 +163,7 @@ static void guess_dc(ERContext *s, int16_t *dc, int w,
         for(b_x=w-1; b_x>=0; b_x--){
             int mb_index_j= (b_x>>is_luma) + (b_y>>is_luma)*s->mb_stride;
             int error_j= s->error_status_table[mb_index_j];
-            int intra_j = IS_INTRA(s->cur_pic->mb_type[mb_index_j]);
+            int intra_j = IS_INTRA(s->cur_pic.mb_type[mb_index_j]);
             if(intra_j==0 || !(error_j&ER_DC_ERROR)){
                 color= dc[b_x + b_y*stride];
                 distance= b_x;
@@ -177,7 +178,7 @@ static void guess_dc(ERContext *s, int16_t *dc, int w,
         for(b_y=0; b_y<h; b_y++){
             int mb_index_j= (b_x>>is_luma) + (b_y>>is_luma)*s->mb_stride;
             int error_j= s->error_status_table[mb_index_j];
-            int intra_j = IS_INTRA(s->cur_pic->mb_type[mb_index_j]);
+            int intra_j = IS_INTRA(s->cur_pic.mb_type[mb_index_j]);
             if(intra_j==0 || !(error_j&ER_DC_ERROR)){
                 color= dc[b_x + b_y*stride];
                 distance= b_y;
@@ -190,7 +191,7 @@ static void guess_dc(ERContext *s, int16_t *dc, int w,
         for(b_y=h-1; b_y>=0; b_y--){
             int mb_index_j= (b_x>>is_luma) + (b_y>>is_luma)*s->mb_stride;
             int error_j= s->error_status_table[mb_index_j];
-            int intra_j = IS_INTRA(s->cur_pic->mb_type[mb_index_j]);
+            int intra_j = IS_INTRA(s->cur_pic.mb_type[mb_index_j]);
             if(intra_j==0 || !(error_j&ER_DC_ERROR)){
                 color= dc[b_x + b_y*stride];
                 distance= b_y;
@@ -207,7 +208,7 @@ static void guess_dc(ERContext *s, int16_t *dc, int w,
             mb_index = (b_x >> is_luma) + (b_y >> is_luma) * s->mb_stride;
             error    = s->error_status_table[mb_index];
 
-            if (IS_INTER(s->cur_pic->mb_type[mb_index]))
+            if (IS_INTER(s->cur_pic.mb_type[mb_index]))
                 continue; // inter
             if (!(error & ER_DC_ERROR))
                 continue; // dc-ok
@@ -238,7 +239,7 @@ static void h_block_filter(ERContext *s, uint8_t *dst, int w,
                            int h, int stride, int is_luma)
 {
     int b_x, b_y, mvx_stride, mvy_stride;
-    const uint8_t *cm = ff_cropTbl + MAX_NEG_CROP;
+    const uint8_t *cm = ff_crop_tab + MAX_NEG_CROP;
     set_mv_strides(s, &mvx_stride, &mvy_stride);
     mvx_stride >>= is_luma;
     mvy_stride *= mvx_stride;
@@ -248,13 +249,13 @@ static void h_block_filter(ERContext *s, uint8_t *dst, int w,
             int y;
             int left_status  = s->error_status_table[( b_x      >> is_luma) + (b_y >> is_luma) * s->mb_stride];
             int right_status = s->error_status_table[((b_x + 1) >> is_luma) + (b_y >> is_luma) * s->mb_stride];
-            int left_intra   = IS_INTRA(s->cur_pic->mb_type[( b_x      >> is_luma) + (b_y >> is_luma) * s->mb_stride]);
-            int right_intra  = IS_INTRA(s->cur_pic->mb_type[((b_x + 1) >> is_luma) + (b_y >> is_luma) * s->mb_stride]);
+            int left_intra   = IS_INTRA(s->cur_pic.mb_type[( b_x      >> is_luma) + (b_y >> is_luma) * s->mb_stride]);
+            int right_intra  = IS_INTRA(s->cur_pic.mb_type[((b_x + 1) >> is_luma) + (b_y >> is_luma) * s->mb_stride]);
             int left_damage  = left_status & ER_MB_ERROR;
             int right_damage = right_status & ER_MB_ERROR;
             int offset       = b_x * 8 + b_y * stride * 8;
-            int16_t *left_mv  = s->cur_pic->motion_val[0][mvy_stride * b_y + mvx_stride *  b_x];
-            int16_t *right_mv = s->cur_pic->motion_val[0][mvy_stride * b_y + mvx_stride * (b_x + 1)];
+            int16_t *left_mv  = s->cur_pic.motion_val[0][mvy_stride * b_y + mvx_stride *  b_x];
+            int16_t *right_mv = s->cur_pic.motion_val[0][mvy_stride * b_y + mvx_stride * (b_x + 1)];
             if (!(left_damage || right_damage))
                 continue; // both undamaged
             if ((!left_intra) && (!right_intra) &&
@@ -306,7 +307,7 @@ static void v_block_filter(ERContext *s, uint8_t *dst, int w, int h,
                            int stride, int is_luma)
 {
     int b_x, b_y, mvx_stride, mvy_stride;
-    const uint8_t *cm = ff_cropTbl + MAX_NEG_CROP;
+    const uint8_t *cm = ff_crop_tab + MAX_NEG_CROP;
     set_mv_strides(s, &mvx_stride, &mvy_stride);
     mvx_stride >>= is_luma;
     mvy_stride *= mvx_stride;
@@ -316,14 +317,14 @@ static void v_block_filter(ERContext *s, uint8_t *dst, int w, int h,
             int x;
             int top_status    = s->error_status_table[(b_x >> is_luma) +  (b_y      >> is_luma) * s->mb_stride];
             int bottom_status = s->error_status_table[(b_x >> is_luma) + ((b_y + 1) >> is_luma) * s->mb_stride];
-            int top_intra     = IS_INTRA(s->cur_pic->mb_type[(b_x >> is_luma) + ( b_y      >> is_luma) * s->mb_stride]);
-            int bottom_intra  = IS_INTRA(s->cur_pic->mb_type[(b_x >> is_luma) + ((b_y + 1) >> is_luma) * s->mb_stride]);
+            int top_intra     = IS_INTRA(s->cur_pic.mb_type[(b_x >> is_luma) + ( b_y      >> is_luma) * s->mb_stride]);
+            int bottom_intra  = IS_INTRA(s->cur_pic.mb_type[(b_x >> is_luma) + ((b_y + 1) >> is_luma) * s->mb_stride]);
             int top_damage    = top_status & ER_MB_ERROR;
             int bottom_damage = bottom_status & ER_MB_ERROR;
             int offset        = b_x * 8 + b_y * stride * 8;
 
-            int16_t *top_mv    = s->cur_pic->motion_val[0][mvy_stride *  b_y      + mvx_stride * b_x];
-            int16_t *bottom_mv = s->cur_pic->motion_val[0][mvy_stride * (b_y + 1) + mvx_stride * b_x];
+            int16_t *top_mv    = s->cur_pic.motion_val[0][mvy_stride *  b_y      + mvx_stride * b_x];
+            int16_t *bottom_mv = s->cur_pic.motion_val[0][mvy_stride * (b_y + 1) + mvx_stride * b_x];
 
             if (!(top_damage || bottom_damage))
                 continue; // both undamaged
@@ -388,7 +389,7 @@ static void guess_mv(ERContext *s)
         int f = 0;
         int error = s->error_status_table[mb_xy];
 
-        if (IS_INTRA(s->cur_pic->mb_type[mb_xy]))
+        if (IS_INTRA(s->cur_pic.mb_type[mb_xy]))
             f = MV_FROZEN; // intra // FIXME check
         if (!(error & ER_MV_ERROR))
             f = MV_FROZEN; // inter with undamaged MV
@@ -396,13 +397,13 @@ static void guess_mv(ERContext *s)
         fixed[mb_xy] = f;
         if (f == MV_FROZEN)
             num_avail++;
-        else if(s->last_pic->f.data[0] && s->last_pic->motion_val[0]){
+        else if(s->last_pic.f->data[0] && s->last_pic.motion_val[0]){
             const int mb_y= mb_xy / s->mb_stride;
             const int mb_x= mb_xy % s->mb_stride;
             const int mot_index= (mb_x + mb_y*mot_stride) * mot_step;
-            s->cur_pic->motion_val[0][mot_index][0]= s->last_pic->motion_val[0][mot_index][0];
-            s->cur_pic->motion_val[0][mot_index][1]= s->last_pic->motion_val[0][mot_index][1];
-            s->cur_pic->ref_index[0][4*mb_xy]      = s->last_pic->ref_index[0][4*mb_xy];
+            s->cur_pic.motion_val[0][mot_index][0]= s->last_pic.motion_val[0][mot_index][0];
+            s->cur_pic.motion_val[0][mot_index][1]= s->last_pic.motion_val[0][mot_index][1];
+            s->cur_pic.ref_index[0][4*mb_xy]      = s->last_pic.ref_index[0][4*mb_xy];
         }
     }
 
@@ -411,9 +412,9 @@ static void guess_mv(ERContext *s)
         for (mb_y = 0; mb_y < s->mb_height; mb_y++) {
             for (mb_x = 0; mb_x < s->mb_width; mb_x++) {
                 const int mb_xy = mb_x + mb_y * s->mb_stride;
-                int mv_dir = (s->last_pic && s->last_pic->f.data[0]) ? MV_DIR_FORWARD : MV_DIR_BACKWARD;
+                int mv_dir = (s->last_pic.f && s->last_pic.f->data[0]) ? MV_DIR_FORWARD : MV_DIR_BACKWARD;
 
-                if (IS_INTRA(s->cur_pic->mb_type[mb_xy]))
+                if (IS_INTRA(s->cur_pic.mb_type[mb_xy]))
                     continue;
                 if (!(s->error_status_table[mb_xy] & ER_MV_ERROR))
                     continue;
@@ -447,15 +448,15 @@ static void guess_mv(ERContext *s)
                     int best_score         = 256 * 256 * 256 * 64;
                     int best_pred          = 0;
                     const int mot_index    = (mb_x + mb_y * mot_stride) * mot_step;
-                    int prev_x, prev_y, prev_ref;
+                    int prev_x = 0, prev_y = 0, prev_ref = 0;
 
                     if ((mb_x ^ mb_y ^ pass) & 1)
                         continue;
 
                     if (fixed[mb_xy] == MV_FROZEN)
                         continue;
-                    av_assert1(!IS_INTRA(s->cur_pic->mb_type[mb_xy]));
-                    av_assert1(s->last_pic && s->last_pic->f.data[0]);
+                    av_assert1(!IS_INTRA(s->cur_pic.mb_type[mb_xy]));
+                    av_assert1(s->last_pic.f && s->last_pic.f->data[0]);
 
                     j = 0;
                     if (mb_x > 0             && fixed[mb_xy - 1]         == MV_FROZEN)
@@ -485,38 +486,38 @@ static void guess_mv(ERContext *s)
 
                     if (mb_x > 0 && fixed[mb_xy - 1]) {
                         mv_predictor[pred_count][0] =
-                            s->cur_pic->motion_val[0][mot_index - mot_step][0];
+                            s->cur_pic.motion_val[0][mot_index - mot_step][0];
                         mv_predictor[pred_count][1] =
-                            s->cur_pic->motion_val[0][mot_index - mot_step][1];
+                            s->cur_pic.motion_val[0][mot_index - mot_step][1];
                         ref[pred_count] =
-                            s->cur_pic->ref_index[0][4 * (mb_xy - 1)];
+                            s->cur_pic.ref_index[0][4 * (mb_xy - 1)];
                         pred_count++;
                     }
                     if (mb_x + 1 < mb_width && fixed[mb_xy + 1]) {
                         mv_predictor[pred_count][0] =
-                            s->cur_pic->motion_val[0][mot_index + mot_step][0];
+                            s->cur_pic.motion_val[0][mot_index + mot_step][0];
                         mv_predictor[pred_count][1] =
-                            s->cur_pic->motion_val[0][mot_index + mot_step][1];
+                            s->cur_pic.motion_val[0][mot_index + mot_step][1];
                         ref[pred_count] =
-                            s->cur_pic->ref_index[0][4 * (mb_xy + 1)];
+                            s->cur_pic.ref_index[0][4 * (mb_xy + 1)];
                         pred_count++;
                     }
                     if (mb_y > 0 && fixed[mb_xy - mb_stride]) {
                         mv_predictor[pred_count][0] =
-                            s->cur_pic->motion_val[0][mot_index - mot_stride * mot_step][0];
+                            s->cur_pic.motion_val[0][mot_index - mot_stride * mot_step][0];
                         mv_predictor[pred_count][1] =
-                            s->cur_pic->motion_val[0][mot_index - mot_stride * mot_step][1];
+                            s->cur_pic.motion_val[0][mot_index - mot_stride * mot_step][1];
                         ref[pred_count] =
-                            s->cur_pic->ref_index[0][4 * (mb_xy - s->mb_stride)];
+                            s->cur_pic.ref_index[0][4 * (mb_xy - s->mb_stride)];
                         pred_count++;
                     }
                     if (mb_y + 1<mb_height && fixed[mb_xy + mb_stride]) {
                         mv_predictor[pred_count][0] =
-                            s->cur_pic->motion_val[0][mot_index + mot_stride * mot_step][0];
+                            s->cur_pic.motion_val[0][mot_index + mot_stride * mot_step][0];
                         mv_predictor[pred_count][1] =
-                            s->cur_pic->motion_val[0][mot_index + mot_stride * mot_step][1];
+                            s->cur_pic.motion_val[0][mot_index + mot_stride * mot_step][1];
                         ref[pred_count] =
-                            s->cur_pic->ref_index[0][4 * (mb_xy + s->mb_stride)];
+                            s->cur_pic.ref_index[0][4 * (mb_xy + s->mb_stride)];
                         pred_count++;
                     }
                     if (pred_count == 0)
@@ -574,19 +575,19 @@ skip_mean_and_median:
                         if (s->avctx->codec_id == AV_CODEC_ID_H264) {
                             // FIXME
                         } else {
-                            ff_thread_await_progress(&s->last_pic->tf,
+                            ff_thread_await_progress(s->last_pic.tf,
                                                      mb_y, 0);
                         }
-                        if (!s->last_pic->motion_val[0] ||
-                            !s->last_pic->ref_index[0])
+                        if (!s->last_pic.motion_val[0] ||
+                            !s->last_pic.ref_index[0])
                             goto skip_last_mv;
-                        prev_x   = s->last_pic->motion_val[0][mot_index][0];
-                        prev_y   = s->last_pic->motion_val[0][mot_index][1];
-                        prev_ref = s->last_pic->ref_index[0][4 * mb_xy];
+                        prev_x   = s->last_pic.motion_val[0][mot_index][0];
+                        prev_y   = s->last_pic.motion_val[0][mot_index][1];
+                        prev_ref = s->last_pic.ref_index[0][4 * mb_xy];
                     } else {
-                        prev_x   = s->cur_pic->motion_val[0][mot_index][0];
-                        prev_y   = s->cur_pic->motion_val[0][mot_index][1];
-                        prev_ref = s->cur_pic->ref_index[0][4 * mb_xy];
+                        prev_x   = s->cur_pic.motion_val[0][mot_index][0];
+                        prev_y   = s->cur_pic.motion_val[0][mot_index][1];
+                        prev_ref = s->cur_pic.ref_index[0][4 * mb_xy];
                     }
 
                     /* last MV */
@@ -598,14 +599,14 @@ skip_mean_and_median:
 skip_last_mv:
 
                     for (j = 0; j < pred_count; j++) {
-                        int *linesize = s->cur_pic->f.linesize;
+                        int *linesize = s->cur_pic.f->linesize;
                         int score = 0;
-                        uint8_t *src = s->cur_pic->f.data[0] +
+                        uint8_t *src = s->cur_pic.f->data[0] +
                                        mb_x * 16 + mb_y * 16 * linesize[0];
 
-                        s->cur_pic->motion_val[0][mot_index][0] =
+                        s->cur_pic.motion_val[0][mot_index][0] =
                             s->mv[0][0][0] = mv_predictor[j][0];
-                        s->cur_pic->motion_val[0][mot_index][1] =
+                        s->cur_pic.motion_val[0][mot_index][1] =
                             s->mv[0][0][1] = mv_predictor[j][1];
 
                         // predictor intra or otherwise not available
@@ -650,8 +651,8 @@ skip_last_mv:
 
                     for (i = 0; i < mot_step; i++)
                         for (j = 0; j < mot_step; j++) {
-                            s->cur_pic->motion_val[0][mot_index + i + j * mot_stride][0] = s->mv[0][0][0];
-                            s->cur_pic->motion_val[0][mot_index + i + j * mot_stride][1] = s->mv[0][0][1];
+                            s->cur_pic.motion_val[0][mot_index + i + j * mot_stride][0] = s->mv[0][0][0];
+                            s->cur_pic.motion_val[0][mot_index + i + j * mot_stride][1] = s->mv[0][0][1];
                         }
 
                     s->decode_mb(s->opaque, ref[best_pred], MV_DIR_FORWARD,
@@ -682,9 +683,12 @@ static int is_intra_more_likely(ERContext *s)
 {
     int is_intra_likely, i, j, undamaged_count, skip_amount, mb_x, mb_y;
 
-    if (!s->last_pic || !s->last_pic->f.data[0])
+    if (!s->last_pic.f || !s->last_pic.f->data[0])
         return 1; // no previous frame available -> use spatial prediction
 
+    if (s->avctx->error_concealment & FF_EC_FAVOR_INTER)
+        return 0;
+
     undamaged_count = 0;
     for (i = 0; i < s->mb_num; i++) {
         const int mb_xy = s->mb_index2xy[i];
@@ -702,7 +706,7 @@ static int is_intra_more_likely(ERContext *s)
     // prevent dsp.sad() check, that requires access to the image
     if (CONFIG_XVMC    &&
         s->avctx->hwaccel && s->avctx->hwaccel->decode_mb &&
-        s->cur_pic->f.pict_type == AV_PICTURE_TYPE_I)
+        s->cur_pic.f->pict_type == AV_PICTURE_TYPE_I)
         return 1;
 
     skip_amount     = FFMAX(undamaged_count / 50, 1); // check only up to 50 MBs
@@ -723,26 +727,26 @@ static int is_intra_more_likely(ERContext *s)
             if ((j % skip_amount) != 0)
                 continue;
 
-            if (s->cur_pic->f.pict_type == AV_PICTURE_TYPE_I) {
-                int *linesize = s->cur_pic->f.linesize;
-                uint8_t *mb_ptr      = s->cur_pic->f.data[0] +
+            if (s->cur_pic.f->pict_type == AV_PICTURE_TYPE_I) {
+                int *linesize = s->cur_pic.f->linesize;
+                uint8_t *mb_ptr      = s->cur_pic.f->data[0] +
                                        mb_x * 16 + mb_y * 16 * linesize[0];
-                uint8_t *last_mb_ptr = s->last_pic->f.data[0] +
+                uint8_t *last_mb_ptr = s->last_pic.f->data[0] +
                                        mb_x * 16 + mb_y * 16 * linesize[0];
 
                 if (s->avctx->codec_id == AV_CODEC_ID_H264) {
                     // FIXME
                 } else {
-                    ff_thread_await_progress(&s->last_pic->tf, mb_y, 0);
+                    ff_thread_await_progress(s->last_pic.tf, mb_y, 0);
                 }
-                is_intra_likely += s->dsp->sad[0](NULL, last_mb_ptr, mb_ptr,
-                                                 linesize[0], 16);
+                is_intra_likely += s->mecc->sad[0](NULL, last_mb_ptr, mb_ptr,
+                                                   linesize[0], 16);
                 // FIXME need await_progress() here
-                is_intra_likely -= s->dsp->sad[0](NULL, last_mb_ptr,
-                                                 last_mb_ptr + linesize[0] * 16,
-                                                 linesize[0], 16);
+                is_intra_likely -= s->mecc->sad[0](NULL, last_mb_ptr,
+                                                   last_mb_ptr + linesize[0] * 16,
+                                                   linesize[0], 16);
             } else {
-                if (IS_INTRA(s->cur_pic->mb_type[mb_xy]))
+                if (IS_INTRA(s->cur_pic.mb_type[mb_xy]))
                    is_intra_likely++;
                 else
                    is_intra_likely--;
@@ -768,8 +772,8 @@ static int er_supported(ERContext *s)
 {
     if(s->avctx->hwaccel && s->avctx->hwaccel->decode_slice           ||
        s->avctx->codec->capabilities&CODEC_CAP_HWACCEL_VDPAU          ||
-       !s->cur_pic                                                    ||
-       s->cur_pic->field_picture
+       !s->cur_pic.f                                                  ||
+       s->cur_pic.field_picture
     )
         return 0;
     return 1;
@@ -854,7 +858,7 @@ void ff_er_add_slice(ERContext *s, int startx, int starty,
 
 void ff_er_frame_end(ERContext *s)
 {
-    int *linesize = s->cur_pic->f.linesize;
+    int *linesize = NULL;
     int i, mb_x, mb_y, error, error_type, dc_error, mv_error, ac_error;
     int distance;
     int threshold_part[4] = { 100, 100, 100 };
@@ -871,6 +875,7 @@ void ff_er_frame_end(ERContext *s)
                           (s->avctx->skip_top + s->avctx->skip_bottom)) {
         return;
     }
+    linesize = s->cur_pic.f->linesize;
     for (mb_x = 0; mb_x < s->mb_width; mb_x++) {
         int status = s->error_status_table[mb_x + (s->mb_height - 1) * s->mb_stride];
         if (status != 0x7F)
@@ -886,40 +891,40 @@ void ff_er_frame_end(ERContext *s)
         return;
     }
 
-    if (s->last_pic) {
-        if (s->last_pic->f.width  != s->cur_pic->f.width  ||
-            s->last_pic->f.height != s->cur_pic->f.height ||
-            s->last_pic->f.format != s->cur_pic->f.format) {
+    if (s->last_pic.f) {
+        if (s->last_pic.f->width  != s->cur_pic.f->width  ||
+            s->last_pic.f->height != s->cur_pic.f->height ||
+            s->last_pic.f->format != s->cur_pic.f->format) {
             av_log(s->avctx, AV_LOG_WARNING, "Cannot use previous picture in error concealment\n");
-            s->last_pic = NULL;
+            memset(&s->last_pic, 0, sizeof(s->last_pic));
         }
     }
-    if (s->next_pic) {
-        if (s->next_pic->f.width  != s->cur_pic->f.width  ||
-            s->next_pic->f.height != s->cur_pic->f.height ||
-            s->next_pic->f.format != s->cur_pic->f.format) {
+    if (s->next_pic.f) {
+        if (s->next_pic.f->width  != s->cur_pic.f->width  ||
+            s->next_pic.f->height != s->cur_pic.f->height ||
+            s->next_pic.f->format != s->cur_pic.f->format) {
             av_log(s->avctx, AV_LOG_WARNING, "Cannot use next picture in error concealment\n");
-            s->next_pic = NULL;
+            memset(&s->next_pic, 0, sizeof(s->next_pic));
         }
     }
 
-    if (s->cur_pic->motion_val[0] == NULL) {
+    if (!s->cur_pic.motion_val[0] || !s->cur_pic.ref_index[0]) {
         av_log(s->avctx, AV_LOG_ERROR, "Warning MVs not available\n");
 
         for (i = 0; i < 2; i++) {
-            s->cur_pic->ref_index_buf[i]  = av_buffer_allocz(s->mb_stride * s->mb_height * 4 * sizeof(uint8_t));
-            s->cur_pic->motion_val_buf[i] = av_buffer_allocz((size + 4) * 2 * sizeof(uint16_t));
-            if (!s->cur_pic->ref_index_buf[i] || !s->cur_pic->motion_val_buf[i])
+            s->ref_index_buf[i]  = av_buffer_allocz(s->mb_stride * s->mb_height * 4 * sizeof(uint8_t));
+            s->motion_val_buf[i] = av_buffer_allocz((size + 4) * 2 * sizeof(uint16_t));
+            if (!s->ref_index_buf[i] || !s->motion_val_buf[i])
                 break;
-            s->cur_pic->ref_index[i]  = s->cur_pic->ref_index_buf[i]->data;
-            s->cur_pic->motion_val[i] = (int16_t (*)[2])s->cur_pic->motion_val_buf[i]->data + 4;
+            s->cur_pic.ref_index[i]  = s->ref_index_buf[i]->data;
+            s->cur_pic.motion_val[i] = (int16_t (*)[2])s->motion_val_buf[i]->data + 4;
         }
         if (i < 2) {
             for (i = 0; i < 2; i++) {
-                av_buffer_unref(&s->cur_pic->ref_index_buf[i]);
-                av_buffer_unref(&s->cur_pic->motion_val_buf[i]);
-                s->cur_pic->ref_index[i]  = NULL;
-                s->cur_pic->motion_val[i] = NULL;
+                av_buffer_unref(&s->ref_index_buf[i]);
+                av_buffer_unref(&s->motion_val_buf[i]);
+                s->cur_pic.ref_index[i]  = NULL;
+                s->cur_pic.motion_val[i] = NULL;
             }
             return;
         }
@@ -1053,7 +1058,7 @@ void ff_er_frame_end(ERContext *s)
     if (!s->partitioned_frame) {
         for (i = 0; i < s->mb_num; i++) {
             const int mb_xy = s->mb_index2xy[i];
-            error = s->error_status_table[mb_xy];
+            int error = s->error_status_table[mb_xy];
             if (error & ER_MB_ERROR)
                 error |= ER_MB_ERROR;
             s->error_status_table[mb_xy] = error;
@@ -1064,7 +1069,7 @@ void ff_er_frame_end(ERContext *s)
     dc_error = ac_error = mv_error = 0;
     for (i = 0; i < s->mb_num; i++) {
         const int mb_xy = s->mb_index2xy[i];
-        error = s->error_status_table[mb_xy];
+        int error = s->error_status_table[mb_xy];
         if (error & ER_DC_ERROR)
             dc_error++;
         if (error & ER_AC_ERROR)
@@ -1073,42 +1078,42 @@ void ff_er_frame_end(ERContext *s)
             mv_error++;
     }
     av_log(s->avctx, AV_LOG_INFO, "concealing %d DC, %d AC, %d MV errors in %c frame\n",
-           dc_error, ac_error, mv_error, av_get_picture_type_char(s->cur_pic->f.pict_type));
+           dc_error, ac_error, mv_error, av_get_picture_type_char(s->cur_pic.f->pict_type));
 
     is_intra_likely = is_intra_more_likely(s);
 
     /* set unknown mb-type to most likely */
     for (i = 0; i < s->mb_num; i++) {
         const int mb_xy = s->mb_index2xy[i];
-        error = s->error_status_table[mb_xy];
+        int error = s->error_status_table[mb_xy];
         if (!((error & ER_DC_ERROR) && (error & ER_MV_ERROR)))
             continue;
 
         if (is_intra_likely)
-            s->cur_pic->mb_type[mb_xy] = MB_TYPE_INTRA4x4;
+            s->cur_pic.mb_type[mb_xy] = MB_TYPE_INTRA4x4;
         else
-            s->cur_pic->mb_type[mb_xy] = MB_TYPE_16x16 | MB_TYPE_L0;
+            s->cur_pic.mb_type[mb_xy] = MB_TYPE_16x16 | MB_TYPE_L0;
     }
 
     // change inter to intra blocks if no reference frames are available
-    if (!(s->last_pic && s->last_pic->f.data[0]) &&
-        !(s->next_pic && s->next_pic->f.data[0]))
+    if (!(s->last_pic.f && s->last_pic.f->data[0]) &&
+        !(s->next_pic.f && s->next_pic.f->data[0]))
         for (i = 0; i < s->mb_num; i++) {
             const int mb_xy = s->mb_index2xy[i];
-            if (!IS_INTRA(s->cur_pic->mb_type[mb_xy]))
-                s->cur_pic->mb_type[mb_xy] = MB_TYPE_INTRA4x4;
+            if (!IS_INTRA(s->cur_pic.mb_type[mb_xy]))
+                s->cur_pic.mb_type[mb_xy] = MB_TYPE_INTRA4x4;
         }
 
     /* handle inter blocks with damaged AC */
     for (mb_y = 0; mb_y < s->mb_height; mb_y++) {
         for (mb_x = 0; mb_x < s->mb_width; mb_x++) {
             const int mb_xy   = mb_x + mb_y * s->mb_stride;
-            const int mb_type = s->cur_pic->mb_type[mb_xy];
-            const int dir     = !(s->last_pic && s->last_pic->f.data[0]);
+            const int mb_type = s->cur_pic.mb_type[mb_xy];
+            const int dir     = !(s->last_pic.f && s->last_pic.f->data[0]);
             const int mv_dir  = dir ? MV_DIR_BACKWARD : MV_DIR_FORWARD;
             int mv_type;
 
-            error = s->error_status_table[mb_xy];
+            int error = s->error_status_table[mb_xy];
 
             if (IS_INTRA(mb_type))
                 continue; // intra
@@ -1122,13 +1127,13 @@ void ff_er_frame_end(ERContext *s)
                 int j;
                 mv_type = MV_TYPE_8X8;
                 for (j = 0; j < 4; j++) {
-                    s->mv[0][j][0] = s->cur_pic->motion_val[dir][mb_index + (j & 1) + (j >> 1) * s->b8_stride][0];
-                    s->mv[0][j][1] = s->cur_pic->motion_val[dir][mb_index + (j & 1) + (j >> 1) * s->b8_stride][1];
+                    s->mv[0][j][0] = s->cur_pic.motion_val[dir][mb_index + (j & 1) + (j >> 1) * s->b8_stride][0];
+                    s->mv[0][j][1] = s->cur_pic.motion_val[dir][mb_index + (j & 1) + (j >> 1) * s->b8_stride][1];
                 }
             } else {
                 mv_type     = MV_TYPE_16X16;
-                s->mv[0][0][0] = s->cur_pic->motion_val[dir][mb_x * 2 + mb_y * 2 * s->b8_stride][0];
-                s->mv[0][0][1] = s->cur_pic->motion_val[dir][mb_x * 2 + mb_y * 2 * s->b8_stride][1];
+                s->mv[0][0][0] = s->cur_pic.motion_val[dir][mb_x * 2 + mb_y * 2 * s->b8_stride][0];
+                s->mv[0][0][1] = s->cur_pic.motion_val[dir][mb_x * 2 + mb_y * 2 * s->b8_stride][1];
             }
 
             s->decode_mb(s->opaque, 0 /* FIXME h264 partitioned slices need this set */,
@@ -1137,15 +1142,15 @@ void ff_er_frame_end(ERContext *s)
     }
 
     /* guess MVs */
-    if (s->cur_pic->f.pict_type == AV_PICTURE_TYPE_B) {
+    if (s->cur_pic.f->pict_type == AV_PICTURE_TYPE_B) {
         for (mb_y = 0; mb_y < s->mb_height; mb_y++) {
             for (mb_x = 0; mb_x < s->mb_width; mb_x++) {
                 int       xy      = mb_x * 2 + mb_y * 2 * s->b8_stride;
                 const int mb_xy   = mb_x + mb_y * s->mb_stride;
-                const int mb_type = s->cur_pic->mb_type[mb_xy];
+                const int mb_type = s->cur_pic.mb_type[mb_xy];
                 int mv_dir = MV_DIR_FORWARD | MV_DIR_BACKWARD;
 
-                error = s->error_status_table[mb_xy];
+                int error = s->error_status_table[mb_xy];
 
                 if (IS_INTRA(mb_type))
                     continue;
@@ -1154,9 +1159,9 @@ void ff_er_frame_end(ERContext *s)
                 if (!(error & ER_AC_ERROR))
                     continue; // undamaged inter
 
-                if (!(s->last_pic && s->last_pic->f.data[0]))
+                if (!(s->last_pic.f && s->last_pic.f->data[0]))
                     mv_dir &= ~MV_DIR_FORWARD;
-                if (!(s->next_pic && s->next_pic->f.data[0]))
+                if (!(s->next_pic.f && s->next_pic.f->data[0]))
                     mv_dir &= ~MV_DIR_BACKWARD;
 
                 if (s->pp_time) {
@@ -1164,12 +1169,12 @@ void ff_er_frame_end(ERContext *s)
                     int time_pb = s->pb_time;
 
                     av_assert0(s->avctx->codec_id != AV_CODEC_ID_H264);
-                    ff_thread_await_progress(&s->next_pic->tf, mb_y, 0);
+                    ff_thread_await_progress(s->next_pic.tf, mb_y, 0);
 
-                    s->mv[0][0][0] = s->next_pic->motion_val[0][xy][0] *  time_pb            / time_pp;
-                    s->mv[0][0][1] = s->next_pic->motion_val[0][xy][1] *  time_pb            / time_pp;
-                    s->mv[1][0][0] = s->next_pic->motion_val[0][xy][0] * (time_pb - time_pp) / time_pp;
-                    s->mv[1][0][1] = s->next_pic->motion_val[0][xy][1] * (time_pb - time_pp) / time_pp;
+                    s->mv[0][0][0] = s->next_pic.motion_val[0][xy][0] *  time_pb            / time_pp;
+                    s->mv[0][0][1] = s->next_pic.motion_val[0][xy][1] *  time_pb            / time_pp;
+                    s->mv[1][0][0] = s->next_pic.motion_val[0][xy][0] * (time_pb - time_pp) / time_pp;
+                    s->mv[1][0][1] = s->next_pic.motion_val[0][xy][1] * (time_pb - time_pp) / time_pp;
                 } else {
                     s->mv[0][0][0] = 0;
                     s->mv[0][0][1] = 0;
@@ -1194,18 +1199,18 @@ void ff_er_frame_end(ERContext *s)
             int16_t *dc_ptr;
             uint8_t *dest_y, *dest_cb, *dest_cr;
             const int mb_xy   = mb_x + mb_y * s->mb_stride;
-            const int mb_type = s->cur_pic->mb_type[mb_xy];
+            const int mb_type = s->cur_pic.mb_type[mb_xy];
 
-            error = s->error_status_table[mb_xy];
+            // error = s->error_status_table[mb_xy];
 
             if (IS_INTRA(mb_type) && s->partitioned_frame)
                 continue;
             // if (error & ER_MV_ERROR)
             //     continue; // inter data damaged FIXME is this good?
 
-            dest_y  = s->cur_pic->f.data[0] + mb_x * 16 + mb_y * 16 * linesize[0];
-            dest_cb = s->cur_pic->f.data[1] + mb_x *  8 + mb_y *  8 * linesize[1];
-            dest_cr = s->cur_pic->f.data[2] + mb_x *  8 + mb_y *  8 * linesize[2];
+            dest_y  = s->cur_pic.f->data[0] + mb_x * 16 + mb_y * 16 * linesize[0];
+            dest_cb = s->cur_pic.f->data[1] + mb_x *  8 + mb_y *  8 * linesize[1];
+            dest_cr = s->cur_pic.f->data[2] + mb_x *  8 + mb_y *  8 * linesize[2];
 
             dc_ptr = &s->dc_val[0][mb_x * 2 + mb_y * 2 * s->b8_stride];
             for (n = 0; n < 4; n++) {
@@ -1247,18 +1252,18 @@ void ff_er_frame_end(ERContext *s)
         for (mb_x = 0; mb_x < s->mb_width; mb_x++) {
             uint8_t *dest_y, *dest_cb, *dest_cr;
             const int mb_xy   = mb_x + mb_y * s->mb_stride;
-            const int mb_type = s->cur_pic->mb_type[mb_xy];
+            const int mb_type = s->cur_pic.mb_type[mb_xy];
 
-            error = s->error_status_table[mb_xy];
+            int error = s->error_status_table[mb_xy];
 
             if (IS_INTER(mb_type))
                 continue;
             if (!(error & ER_AC_ERROR))
                 continue; // undamaged
 
-            dest_y  = s->cur_pic->f.data[0] + mb_x * 16 + mb_y * 16 * linesize[0];
-            dest_cb = s->cur_pic->f.data[1] + mb_x *  8 + mb_y *  8 * linesize[1];
-            dest_cr = s->cur_pic->f.data[2] + mb_x *  8 + mb_y *  8 * linesize[2];
+            dest_y  = s->cur_pic.f->data[0] + mb_x * 16 + mb_y * 16 * linesize[0];
+            dest_cb = s->cur_pic.f->data[1] + mb_x *  8 + mb_y *  8 * linesize[1];
+            dest_cr = s->cur_pic.f->data[2] + mb_x *  8 + mb_y *  8 * linesize[2];
 
             put_dc(s, dest_y, dest_cb, dest_cr, mb_x, mb_y);
         }
@@ -1267,19 +1272,19 @@ void ff_er_frame_end(ERContext *s)
 
     if (s->avctx->error_concealment & FF_EC_DEBLOCK) {
         /* filter horizontal block boundaries */
-        h_block_filter(s, s->cur_pic->f.data[0], s->mb_width * 2,
+        h_block_filter(s, s->cur_pic.f->data[0], s->mb_width * 2,
                        s->mb_height * 2, linesize[0], 1);
-        h_block_filter(s, s->cur_pic->f.data[1], s->mb_width,
+        h_block_filter(s, s->cur_pic.f->data[1], s->mb_width,
                        s->mb_height, linesize[1], 0);
-        h_block_filter(s, s->cur_pic->f.data[2], s->mb_width,
+        h_block_filter(s, s->cur_pic.f->data[2], s->mb_width,
                        s->mb_height, linesize[2], 0);
 
         /* filter vertical block boundaries */
-        v_block_filter(s, s->cur_pic->f.data[0], s->mb_width * 2,
+        v_block_filter(s, s->cur_pic.f->data[0], s->mb_width * 2,
                        s->mb_height * 2, linesize[0], 1);
-        v_block_filter(s, s->cur_pic->f.data[1], s->mb_width,
+        v_block_filter(s, s->cur_pic.f->data[1], s->mb_width,
                        s->mb_height, linesize[1], 0);
-        v_block_filter(s, s->cur_pic->f.data[2], s->mb_width,
+        v_block_filter(s, s->cur_pic.f->data[2], s->mb_width,
                        s->mb_height, linesize[2], 0);
     }
 
@@ -1289,13 +1294,21 @@ ec_clean:
         const int mb_xy = s->mb_index2xy[i];
         int       error = s->error_status_table[mb_xy];
 
-        if (s->cur_pic->f.pict_type != AV_PICTURE_TYPE_B &&
+        if (s->cur_pic.f->pict_type != AV_PICTURE_TYPE_B &&
             (error & (ER_DC_ERROR | ER_MV_ERROR | ER_AC_ERROR))) {
             s->mbskip_table[mb_xy] = 0;
         }
         s->mbintra_table[mb_xy] = 1;
     }
-    s->cur_pic = NULL;
-    s->next_pic    = NULL;
-    s->last_pic    = NULL;
+
+    for (i = 0; i < 2; i++) {
+        av_buffer_unref(&s->ref_index_buf[i]);
+        av_buffer_unref(&s->motion_val_buf[i]);
+        s->cur_pic.ref_index[i]  = NULL;
+        s->cur_pic.motion_val[i] = NULL;
+    }
+
+    memset(&s->cur_pic, 0, sizeof(ERPicture));
+    memset(&s->last_pic, 0, sizeof(ERPicture));
+    memset(&s->next_pic, 0, sizeof(ERPicture));
 }
