@@ -73,7 +73,7 @@ enum var_name {
     VAR_VARS_NB
 };
 
-typedef struct {
+typedef struct CropContext {
     const AVClass *class;
     int  x;             ///< x offset of the non-cropped area with respect to the input area
     int  y;             ///< y offset of the non-cropped area with respect to the input area
@@ -95,7 +95,7 @@ static int query_formats(AVFilterContext *ctx)
     AVFilterFormats *formats = NULL;
     int fmt;
 
-    for (fmt = 0; fmt < AV_PIX_FMT_NB; fmt++) {
+    for (fmt = 0; av_pix_fmt_desc_get(fmt); fmt++) {
         const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(fmt);
         if (!(desc->flags & (AV_PIX_FMT_FLAG_HWACCEL | AV_PIX_FMT_FLAG_BITSTREAM)) &&
             !((desc->log2_chroma_w || desc->log2_chroma_h) && !(desc->flags & AV_PIX_FMT_FLAG_PLANAR)))
@@ -332,7 +332,7 @@ static const AVFilterPad avfilter_vf_crop_outputs[] = {
     { NULL }
 };
 
-AVFilter avfilter_vf_crop = {
+AVFilter ff_vf_crop = {
     .name          = "crop",
     .description   = NULL_IF_CONFIG_SMALL("Crop the input video."),
     .priv_size     = sizeof(CropContext),
