@@ -269,7 +269,7 @@ AVOutputFormat ff_mlp_muxer = {
 AVOutputFormat ff_mpeg1video_muxer = {
     .name              = "mpeg1video",
     .long_name         = NULL_IF_CONFIG_SMALL("raw MPEG-1 video"),
-    .mime_type         = "video/x-mpeg",
+    .mime_type         = "video/mpeg",
     .extensions        = "mpg,mpeg,m1v",
     .audio_codec       = AV_CODEC_ID_NONE,
     .video_codec       = AV_CODEC_ID_MPEG1VIDEO,
